import Sequelize from 'sequelize'
import dotenv from 'dotenv'
import User from './User.js'
import Post from './Post.js'

dotenv.config()

const sequelize = new Sequelize(
  process.env.POSTGRES_NAME,
  process.env.POSTGRES_USER,
  process.env.POSTGRES_PASSWORD,
  {
    host: process.env.POSTGRES_HOST,
    dialect: 'postgres',
    port: process.env.POSTGRES_PORT,
    underscored: true,
    logging: false,
    pool: {
      max: 5,
      min: 0,
      acquire: 30000,
      idle: 10000,
    },
    define: {
      freezeTableName: true,
    },
  }
)

const models = {
  User: User.init(sequelize, Sequelize),
  Post: Post.init(sequelize, Sequelize),
}

try {
  sequelize.authenticate()
  console.log('Connection has been established successfully.')
} catch (error) {
  console.log('Unable to connect to the database:', error)
}

Object.keys(models).forEach((modelName) => {
  if ('associate' in models[modelName]) {
    models[modelName].associate(models)
  }
})
models.sequelize = sequelize
models.Sequelize = Sequelize

export { sequelize, models }
