import express from 'express'
import bodyParser from 'body-parser'
import cors from 'cors'
import http from 'http'
import { gql } from 'apollo-server'
import { buildSubgraphSchema } from '@apollo/federation'
import { applyMiddleware } from 'graphql-middleware'
import { ApolloServer } from 'apollo-server-express'

export const initializeServer = async (
  models,
  typeDefs,
  resolvers,
  middlewares
) => {
  const schemaWithoutMiddleware = buildSubgraphSchema([
    {
      typeDefs: gql`
        ${typeDefs}
      `,
      resolvers: { ...resolvers },
      resolverValidationOptions: { requireResolversForResolveType: false },
    },
  ])

  const schema = applyMiddleware(schemaWithoutMiddleware, ...middlewares)

  const server = new ApolloServer({
    schema,
    context: async (ctx) => ({
      models,
      req: ctx.req,
    }),
  })

  const corsOptions = {
    credentials: true,
    origin: true,
  }

  const app = express()
  app.use(express.json())
  app.use(cors(corsOptions))
  app.use(bodyParser.urlencoded({ extended: false }))
  app.use(bodyParser.json())

  await server.start()
  server.applyMiddleware({ app, cors: false })

  const httpServer = http.createServer(app)

  models.sequelize.sync({}).then(() => {
    httpServer.listen(process.env.PORT, () => {
      console.log(
        `🚀 Server ready at http://localhost:${process.env.PORT}${server.graphqlPath}`
      )
    })
  })
}
